<?php
/**
* The image-object
*
* Provides functionality for adding images to the page
*
* @author	Alexander Wirtz <alex@pc4p.net>
* @package	pc4p
* @version	$Id: pc4p_image.inc,v 1.33 2002/08/12 10:50:39 eru Exp $
*/
class pc4p_image extends pc4p_object
{
	/**
	* Margins for the image
	*
	* @var		array	$margin
	* @access	private
	*/
	var $margin = array( "top" => 0, "bottom" => 0, "left" => 0, "right" => 0 );

	/**
	* Weblink for the object
	*
	* @var		array	$weblink
	* @access	private
	*/
	var $weblink = array( "URI" => "", "style" => "solid", "width" => 1, "color" => array( 0, 0, 0 ) );

	/**
	* Pointer to the image
	*
	* @var		integer	$pim
	* @access	private
	*/
	var $pim;

	/**
	* Name or resource of the image
	*
	* @var		mixed	$imgfile
	* @access	private
	*/
	var $imgfile;

	/**
	* Type of the image
	*
	* @var		string	$imgtype
	* @access	private
	*/
	var $imgtype;

	/**
	* Scale of the image
	*
	* @var		float	$imgscale
	* @access	private
	*/
	var $imgscale = 1.0;

	/**
	*pc4p_image constructor
	*
	* Calls the constructor of pc4p_object
	*
	* @param	object	pc4p_page	&$parent	Parent of the image
	* @access	public
	* @see		pc4p_object::pc4p_object()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_image( &$parent )
	{
		pc4p_object::pc4p_object( $parent );
	}

	/**
	* Sets the imagescale for this object.
	*
	* Needed for resizing the image
	*
	* @param	float	$scale	Accepts any float (example: 0.5 is half the size)
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_imagescale( $scale )
	{
		$this->imgscale = $scale;
	}

	/**
	* Sets a weblink for this object.
	*
	* Add a weblink to this object, that will be opened on clicking on this object
	*
	* @param	string	$weblink	The URI of the weblink
	* @param	string	$style		The style of the border representing the weblink; "solid" or "dashed"
	* @param	integer	$width		The width of the border representing the weblink; 0 represents an invisible border
	* @param	string	$color		The color of the border representing the weblink; you can use one of the 16 standardcolors from HTML or the hex-notation (#rrggbb)
	* @access	public
	* @see		pc4p_main::pc4p_check_color()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_weblink( $weblink, $style = "", $width = -1, $color = "" )
	{
		if( !empty( $weblink ) ) {
			// Not much to be said here, we check for valid parameters and set them in the object
			$this->weblink[ "URI" ] = $weblink;
			if( !empty( $style ) && ( $style == "solid" || $style == "dashed" ) ) {
				$this->weblink[ "style" ] = $style;
			}
			if( is_numeric( $width ) && $width >= 0 ) {
				$this->weblink[ "width" ] = $width;
			}
			if( !empty( $color ) ) {
				$this->weblink[ "color" ] = pc4p_main::pc4p_check_color( $color, "line" );
			}
		}
	}

	/**
	* Sets the image for this object.
	*
	* To set the image-name, you'll call this function.
	* Checks, if you have a valid image-type
	*
	* @param	mixed	$image	The name of the image or a valid resource of a PHP-created image
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_image( $image )
	{
		// Check, if we have a resource, or a string as filename
		if( is_resource( $image ) ) {
			// It is a resource, now check, if it's an image-resource
			if( imagesx( $image ) > 0 || imagesy( $image ) > 0 ) {
				// Jep, so be it!
				$this->imgfile = $image;
			}
			else {
				die( "Error: ".get_class( $this )."(".__LINE__.") - the resource of this image is not valid." );
			}
		}
		else {
			// It's a filename, check for correct and supported image-type
			$imgtype = strtolower( substr( strrchr( $image, "." ), 1 ) );
			switch ( $imgtype ) {
				case "png" :	$this->imgtype = "png";
								break;
				case "gif" :	$this->imgtype = "gif";
								break;
				case "tiff" :
				case "tif" :	$this->imgtype = "tiff";
								break;
				case "jpeg" :
				case "jpg" :	$this->imgtype = "jpeg";
								break;
				default:		die( "Error: ".get_class( $this )."(".__LINE__.") - this imagefiletype is not supported: ".$imgtype."." );
			}
			if ( file_exists( $image ) ) {
				// Everything's cool
				$this->imgfile = $image;
			}
			else {
				die( "Error: ".get_class( $this )."(".__LINE__.") - image not found: ".$image."." );
			}
		}
	}

	/**
	* Draw-function for the image
	*
	* Opens the image file, rotates the coordinate-system if neccessary and places
	* the image on the page.
	*
	* @access	private
	* @see		pc4p_object::pc4p_calc_rot(), pc4p_object::pc4p_calc_rot_dim()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw()
	{
		if( is_resource( $this->imgfile ) ) {
			$this->pim = pdf_open_memory_image( $this->pdfp, $this->imgfile );
		}
		else {
			$this->pim = pdf_open_image_file( $this->pdfp, $this->imgtype, $this->imgfile );
		}

		// Calculate the coordinates in case of a rotation...straightforward matrix-stuff
		$rot_dim = $this->pc4p_calc_rot_dim( $this->width, $this->height, $this->phi );
		$transp  = $this->pc4p_calc_rot( $this->draw_x + $rot_dim[ "width" ] * $this->imgscale / 2.0, -( $this->draw_y + $rot_dim[ "height" ] * $this->imgscale / 2.0 ), -$this->phi );

		// Rotate the image
		pdf_save( $this->pdfp );
		pdf_rotate( $this->pdfp, $this->phi );
		pdf_place_image( $this->pdfp, $this->pim, $transp[ "x" ] - $this->width * $this->imgscale / 2.0, $transp[ "y" ] - $this->height * $this->imgscale / 2.0, $this->imgscale );
		pdf_restore( $this->pdfp );

		// The handling of the pdf_add_weblink-func is crappy, because it doesn't honor a pdf_translate. Everything has to be calculated manually :-(
		if( !empty( $this->weblink[ "URI" ] ) ) {
			pdf_set_border_style( $this->pdfp, $this->weblink[ "style" ], $this->weblink[ "width" ] );
			pdf_set_border_color( $this->pdfp, $this->weblink[ "color" ][ 0 ], $this->weblink[ "color" ][ 1 ], $this->weblink[ "color" ][ 2 ] );
			pdf_add_weblink( $this->pdfp, $this->draw_x, $this->pageheight - $this->draw_y, $this->draw_x + $rot_dim[ "width" ] * $this->imgscale, $this->pageheight - $this->draw_y - $rot_dim[ "height" ] * $this->imgscale, $this->weblink[ "URI" ] );
		}

		pdf_close_image( $this->pdfp, $this->pim );

		unset( $this->pdfobj );
	}

	/**
	* Calculates the dimensions for the image
	*
	* Checks, if the width of the image doesn't exceed the
	* parent width. If yes, it changes the scale.
	* Calculates the rotation-dimensions, if neccessary.
	* Sets the alignment of the image
	*
	* @param	object	pc4p_page	&$parent	Parent of the image
	* @return	integer	$this->height	Calculated height of the image
	* @access	private
	* @see		pc4p_object::pc4p_calc_rot_dim()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_offset( &$parent )
	{
		if ( get_class( $parent ) == "pc4p_page" ) {
			$this->pagewidth = $parent->width;
			$this->pageheight = $parent->height;
		}
		else {
			$this->pagewidth = $parent->pagewidth;
			$this->pageheight = $parent->pageheight;
		}

		if( is_resource( $this->imgfile ) ) {
			$this->pim = pdf_open_memory_image( $this->pdfp, $this->imgfile );
		}
		else {
			$this->pim = pdf_open_image_file( $this->pdfp, $this->imgtype, $this->imgfile );
		}

		$this->width = $parent->width - ( $this->margin[ "left" ] + $this->margin[ "right" ] + $parent->margin[ "left" ] + $parent->margin[ "right" ] );

		$this->act_x = $parent->draw_x;
		$this->act_y = $parent->draw_y;

		// Check, if the image * scale is bigger than the width of the parent
		// If yes, calculate new scale
		$imgwidth = pdf_get_value( $this->pdfp, "imagewidth", $this->pim );
		$imgheight = pdf_get_value( $this->pdfp, "imageheight", $this->pim );
		$rot_dim = $this->pc4p_calc_rot_dim( $imgwidth, $imgheight, $this->phi );
		if ( ( $rot_dim[ "width" ] * $this->imgscale ) > $this->width ) {
			$this->imgscale = $this->width / $rot_dim[ "width" ];
		}
		$rot_dim[ "width" ] *= $this->imgscale;
		$rot_dim[ "height" ] *= $this->imgscale;

		// Set Alignment accordingly
		if ( $this->alignment == "center" ) {
			$this->draw_x = $this->act_x + $this->margin[ "left" ] + ( $this->width - $rot_dim[ "width" ] ) / 2;
		}
		elseif( $this->alignment == "right" ) {
			$this->draw_x = $this->act_x + $this->margin[ "left" ] + $this->width - $rot_dim[ "width" ];
		}
		else
		{
			$this->draw_x = $this->act_x + $this->margin[ "left" ];
		}

		$this->draw_y = $this->act_y + $this->margin[ "top" ];

		$this->width = $imgwidth;
		$this->height = $imgheight;

		pdf_close_image( $this->pdfp, $this->pim );
		return $this->margin[ "top" ] + $rot_dim[ "height" ] + $this->margin[ "bottom" ];
	}
}
?>
