<?php
/**
* PC4P's main class
*
* In this object, everything concerning the main setup of a PDF is contained. Most important is the pdf-pointer, of course, which is handled down to every child in this PDF.
*
* @author	Alexander Wirtz <alex@pc4p.net>
* @package	pc4p
* @version	$Id: pc4p_main.inc,v 1.50 2002/08/12 10:50:39 eru Exp $
*/
class pc4p_main
{
	/**
	* Array of all children of this object
	*
	* @var		array	$children
	* @access	private
	*/
	var $children = array();

	/**
	* PDF-Pointer used for all pdf_*
	*
	* @var		integer	$pdfp
	* @access	private
	*/
	var $pdfp;

	/**
	* Is the PDF opened?
	*
	* @var		boolean	$pdf_opened
	* @access	private
	*/
	var $pdf_opened = false;

	/**
	* Name of the PDF-File
	*
	* @var		string	$filename
	* @access	private
	*/
	var $filename = "pc4p.pdf";

	/**
	* Shall the PDF be saved, or viewed?
	*
	* @var		string	$save_pdf
	* @access	private
	*/
	var $save_pdf = "i";

	/**
	* Path to the tempfile
	*
	* @var		string	$tempfile
	* @access	private
	*/
	var $tempfile = "";

	/**
	* Pointer to the current page, used when walking through $children
	*
	* @var		object	pc4p_page	$curr_page
	* @access	private
	*/
	var $curr_page;

	/**
	* Pointer to the next page, used when walking through $children
	*
	* @var		object	pc4p_page	$next_page
	* @access	private
	*/
	var $next_page;

	/**
	* Should PC4P always create a new page on pagebreak?
	*
	* @var		integer	$newpage
	* @access	private
	*/
	var $newpage = 0;

	/**
	* Saves the info-array, if we're still in object scope
	*
	* @var		array $info_array
	* @access	private
	*/
	var $info_array;

	/**
	* Saves the compression, if we're still in object scope
	*
	* @var		integer	$compression
	* @access	private
	*/
	var $compression;

	/**
	* As there are no static members, I use a central array to store the fonts
	*
	* @var		array	$fonts
	* @access	private
	*/
	var $fonts = array();

	/**
	* As there are no static members, I use a central array to store the already allocated bookmarks
	*
	* @var		array	$bookmarks
	* @access	private
	*/
	var $bookmarks = array( 0 => 0 );

	/**
	* As there are no static members, I use a central variable to store the current bookmark identifier
	*
	* @var		integer	$bookmark_identifier
	* @access	private
	*/
	var $bookmark_identifier = 1;

	/**
	* pc4p_main constructor
	*
	* Initializes the filepointer required for apprehending the PDF-Pointer, which is initialized also
	*
	* @access	private
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_main()
	{
		// Acquire PDF-Pointer
		$this->pdfp = pdf_new();
	}

	/**
	* Opens the PDF object
	*
	* Gets called from pc4p_page, opens a PDF object in buffer or as a file, sets the info and compression
	*
	* @access	private
	* @see		pc4p_main::pc4p_set_info()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_open_pdf()
	{
		$this->pdf_opened = true;
		// Create empty document
		pdf_open_file( $this->pdfp, $this->tempfile );
		// Predefine the creator of this PDF (hehe)
		pdf_set_info( $this->pdfp, "Creator", "PC4P - PDF Class for PHP" );

		if ( isset( $this->info_array ) ) {
			$this->pc4p_set_info( $this->info_array );
		}
		if ( isset( $this->compression ) ) {
			pdf_set_value( $this->pdfp, "compress", $this->compression );
		}
	}

	/**
	* Check if a color is standard or custom
	*
	* This function checks, if the color is one of the 16 standardcolors from HTML or if it is in the hex-notation #rrggbb. If the color fits one of these schemes, the color is returned, otherwise the function returns black.
	*
	* @param	string	$color	You can use one of the 16 standardcolors from HTML or the hex-notation (#rrggbb)
	* @param	string	$type	What do you want to check, linecolor or fillcolor? Different returntype if color isn't found!!! (Input: "line" or "fill" )
	* @return	mixed	$color	The color in percentage of each colorchannel
	* @access	private
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_check_color( $color = "", $type = "line" )
	{
		$colors = array(	"black"		=> "#000000",
							"maroon"	=> "#800000",
							"green"		=> "#008000",
							"olive"		=> "#808000",
							"navy"		=> "#000080",
							"purple"	=> "#800080",
							"teal"		=> "#008080",
							"gray"		=> "#808080",
							"silver"	=> "#C0C0C0",
							"red"		=> "#FF0000",
							"lime"		=> "#00FF00",
							"yellow"	=> "#FFFF00",
							"blue"		=> "#0000FF",
							"fuchsia"	=> "#FF00FF",
							"aqua"		=> "#00FFFF",
							"white"		=> "#FFFFFF" );

		// Check if we have a standard-color here
		if ( in_array( strtolower( $color ), array_flip( $colors ) ) ) {
			$color = $colors[ strtolower( $color ) ];
		}

		// Now test if the color is in hex-notation
		if ( preg_match( "/^\#?([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})$/i", $color, $colorvals ) ) {
			array_shift( $colorvals );
			foreach ( $colorvals as $ckey => $cval ) {
				$colorvals[ $ckey ] = ( hexdec( $cval ) / 255.0 );
			}
			return $colorvals;
		}
		else {
			if ( $type == "line" ) {
				return array( 0, 0, 0 );
			}
			else {
				return "";
			}
		}
	}

	/**
	* Set the behaviour of the output
	*
	* If you want to make a direct save, or if you want to change the filename, this is the function you should use
	*
	* @param	string	$save_pdf	"inline", "attached", "drive", or the first char of these
	* @param	string	$filename	When saving on a drive, with leading directories, else without
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_savepdf( $save_pdf, $filename = "" )
	{
		if ( $save_pdf == "attached" || $save_pdf == "a" ) {
			$this->save_pdf = "a";
		}
		elseif ( $save_pdf == "drive" || $save_pdf == "d" ) {
			$this->save_pdf = "d";
		}
		else {
			$this->save_pdf = "i";
		}
		if ( !empty( $filename ) ) {
			if ( $this->save_pdf == "a" || $this->save_pdf == "i" ) {
				$this->filename = basename( $filename );
			}
			elseif ( $this->save_pdf == "d" && strlen( dirname( $filename ) ) == 0 ) {
				die( "Error: ".get_class( $this )."(".__LINE__.") - please provide a complete filename with a path, when saving to a drive." );
			}
			else {
				$this->filename = $filename;
			}
		}
	}

	/**
	* Changes the behaviour after a pagebreak
	*
	* Sets if a new page after a pagebreak shall be created, or if an already created page shall be used
	*
	* @param	mixed	$newpage	"yes", "y", 1 - take your pick, anything that looks true will force a newpage
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_newpage( $newpage )
	{
		if ( $newpage > 0 || $newpage == "yes" || $newpage == "y" ) {
			$this->newpage = 1;
		}
		else {
			$this->newpage = 0;
		}
	}

	/**
	* Draw-Function for the PDF
	*
	* Calls the draw for each child, closes PDF Document, then makes a get_buffer on the PDF-Document, to hand it to the browser. If PC4PDEBUG is set, the Object-Tree is printed. Called as last function in the class.
	*
	* @access	public
	* @see		pc4p_main::pc4p_draw_children()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw()
	{
		// Draw the object of this PDF
		$start = explode( " ", microtime() );
		$this->pc4p_draw_children();
		$end = explode( " ", microtime() );

		if ( !defined( "PC4PDEBUG" ) ) {
			// Close PDF object and get buffer or content of the temporary file...
			pdf_close( $this->pdfp );
			if ( $this->tempfile == "" ) {
				$pdfdata = pdf_get_buffer( $this->pdfp );
			}
			else {
				$fd = fopen ( $this->tempfile, "rb" );
				$pdfdata = fread ( $fd, filesize ($this->tempfile ) );
				fclose ( $fd );
			}

			if ( $this->save_pdf == "i" ) {
				// Inline display
				header( "Content-Type: application/pdf; name=".$this->filename );
				header( "Content-Length: " . strlen( $pdfdata ) );
				header( "Content-Disposition: inline; filename=".$this->filename );
				echo $pdfdata;
			}
			elseif( $this->save_pdf == "a" ) {
				// Force browser to open a "Save as" window...IE has problems here
				if ( strstr( getenv( "HTTP_USER_AGENT" ), "MSIE" ) ) {
					header( "Content-Type: application/pdf; name=".$this->filename );
				}
				else {
					header( "Content-Type: application/download; name=".$this->filename );
				}
				header( "Content-Length: " . strlen( $pdfdata ) );
				header( "Content-Disposition: attachment; filename=".$this->filename );
				echo $pdfdata;
			}
			else {
				// Save the created PDF on disk
				$fp = @fopen( $this->filename, "wb" );
				if ( !$fp ) {
					die( "Error: ".get_class( $this )."(".__LINE__.") - couldn't save ".$this->filename.", maybe you don't have the necessary permissions?" );
				}
				fwrite( $fp, $pdfdata, strlen( $pdfdata ) );
				fclose( $fp );
			}
		}
		else {
			// Output the complete object-tree to the browser...might take some time, can get quite large
			echo "<PRE>\n";
			var_dump( $this );
			$time = ( (float) $end[ 0 ] + (float) $end[ 1 ] ) - ( (float) $start[ 0 ] + (float) $start[ 1 ] );
			echo "\nProcessing-time: ".number_format( $time, 4 )."s\n";
			echo "</PRE>\n";
			pdf_close( $this->pdfp );
			pdf_get_buffer( $this->pdfp );
		}
		pdf_delete( $this->pdfp );

		if ( $this->tempfile != "" ) {
			unlink( $this->tempfile );
		}
	}

	/**
	* Topmost draw-children function
	*
	* Calls the draw function for each child in the children-array
	*
	* @access	private
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw_children()
	{
		for ( $i = 0; $i < sizeof( $this->children ); $i++ ) {
			// Get the next child
			unset( $this->next_page );
			$this->curr_page = &$this->children[ $i ];
			// If a next page is there, set member accordingly...this is used in the
			// pagebreak procedure in pc4p_page
			if ( isset( $this->children[ $i + 1 ] ) && is_object( $this->children[ $i + 1 ] ) ) {
				$this->next_page = &$this->children[ $i + 1 ];
			}
			else {
				$this->next_page = "";
			}
			// Calculate the current page
			$this->curr_page->pagenum = $i + 1;
			$this->curr_page->pc4p_calc_offset( $this );
			$this->curr_page->pc4p_draw( $this );
		}
		unset( $this->curr_page );
		unset( $this->next_page );
	}

	/**
	* Sets the info values for the PDF
	*
	* Here, you can handle some general information to the PDF. You can have a look at it, if you use the Acrobat-Reader and select the general options of a file.
	*
	* @param	array	$info_array	Contains the fields and their values. Standard Fields: Subject, Title, Creator, Author, Keywords
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_info( $info_array )
	{
		$standard_values = array( "subject", "title", "creator", "author", "keywords", "trapped" );
		if ( $this->pdf_opened ) {
			// Walk through the array and apply the set_info for each pair
			foreach ( $info_array as $fieldname => $value ) {
				// Check for possible Values and set value if valid
				if ( strlen( $value ) < 256 ) {
					if ( in_array( strtolower( $fieldname ), $standard_values ) ) {
						pdf_set_info( $this->pdfp, ucfirst( strtolower( $fieldname ) ), $value );
					}
					elseif ( $fieldname == strtr( $fieldname, " %()<>[]{}/#", "............" ) ) {
						pdf_set_info( $this->pdfp, $fieldname, $value );
					}
				}
			}
		}
		else {
			$this->info_array = $info_array;
		}
	}

	/**
	* Sets the resourcefile
	*
	* If you need customized fonts, you must define a resourcefile, usually it's called pdflib.upr
	*
	* @param	string	$rfile	Complete path for the resourcefile
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_resourcefile( $rfile )
	{
		if ( !$this->pdf_opened ) {
			if ( file_exists( $rfile) && is_file( $rfile ) ) {
				pdf_set_parameter( $this->pdfp, "resourcefile", $rfile );
			}
			else {
				die( "Error: ".get_class( $this )."(".__LINE__.") - ".$resourcefile." is not a valid file." );
			}
		}
	}

	/**
	* Sets the version of Acrobat
	*
	* This function controls the version of Acrobat you will use in this PDF. You can set it to version 3, 4 or 5.
	*
	* @param	string	$comp	"1.2", "1.3" or "1.4" for Acrobat 3, 4 or 5
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_compatibility( $comp = "1.3" )
	{
		if ( !in_array( $comp, array( "1.2", "1.3", "1.4" ) ) ) {
			die( "Error: ".get_class( $this )."(".__LINE__.") - ".$comp." is not a valid PDF version." );
		}
		if ( !$this->pdf_opened ) {
			pdf_set_parameter( $this->pdfp, "compatibility", $comp );
		}
	}

	/**
	* Enables PC4P to use a temporyfile not the memory for PDF creation
	*
	* Use this function to create your PDFs with temporary files. Function expects a valid path, where you have write-access
	*
	* @param	string	$path
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_tempfile( $path = "/tmp" )
	{
		if ( !$this->pdf_opened ) {
			$path = dirname( $path );
			if ( !( $this->tempfile = tempnam( $path, "PC4P" ) ) ) {
				die( "Error: ".get_class( $this )."(".__LINE__.") - Cannot create temporary file in ".$path."." );
			}
		}
	}

	/**
	* Sets the serial string
	*
	* If you have bought a license of the PDFlib, you can set the serial with this function
	*
	* @param	string	$serial	The serial for your copy of the PDFlib
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_serial( $serial )
	{
		if ( !$this->pdf_opened ) {
			pdf_set_parameter( $this->pdfp, "serial", $serial );
		}
	}

	/**
	* Sets the compression-parameter
	*
	* PDFs with versionnumber >= 4 can have a compression-factor. Use this function to enable compression for your PDF.
	*
	* @param	integer	$compression	Valid range is from 0 to 9
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_compression( $compression )
	{
		if ( is_int( $compression ) ) {
			if ( $compression < 0 ) {
				$compression = 0;
			}
			elseif ( $compression > 9 ) {
				$compression = 9;
			}
			if ( $this->pdf_opened ) {
				pdf_set_value( $this->pdfp, "compress", $compression );
			}
			else {
				$this->compression = $compression;
			}
		}
		else {
			die( "Error: ".get_class( $this )."(".__LINE__.") - ".$compression." is not a valid compression-factor." );
		}
	}
}
?>
