<?php
/**
* The master object for content-objects
*
* In this object, the general object-handling functions and variables are defined. As this is the superclass for the other objects, all functions and variables here can be overwritten.
*
* @author	Alexander Wirtz <alex@pc4p.net>
* @package	pc4p
* @version	$Id: pc4p_object.inc,v 1.48 2002/08/12 10:50:39 eru Exp $
*/
class pc4p_object
{
	/**
	* Array of all children of this object
	*
	* @var		array	$children
	* @access	private
	*/
	var $children = array();

	/**
	* PDF-Pointer used for all pdf_*
	*
	* @var		integer	$pdfp
	* @access	private
	*/
	var $pdfp;

	/**
	* The main PDF-Object
	*
	* @var		object pc4p_main	$pdfobj
	* @access	private
	*/
	var $pdfobj;

	/**
	* Width of the page the object is on
	*
	* @var		integer	$pagewidth
	* @access	private
	*/
	var $pagewidth;

	/**
	* Height of the page the object is on
	*
	* @var		integer	$pageheight
	* @access	private
	*/
	var $pageheight;

	/**
	* Width of the object
	*
	* @var		integer	$width
	* @access	private
	*/
	var $width;

	/**
	* Height of the object
	*
	* @var		integer	$height
	* @access	private
	*/
	var $height;

	/**
	* Margins for the object
	*
	* @var		array	$margin
	* @access	private
	*/
	var $margin = array( "top" => 0, "bottom" => 0, "left" => 0, "right" => 0 );

	/**
	* x-Pos where the object is placed
	*
	* @var		integer	$act_x
	* @access	private
	*/
	var $act_x;

	/**
	* y-Pos where the object is placed
	*
	* @var		integer	$act_y
	* @access	private
	*/
	var $act_y;

	/**
	* x-Pos used when calculating the positions of subobjects
	*
	* @var		integer	$draw_x
	* @access	private
	*/
	var $draw_x;

	/**
	* y-Pos used when calculating the positions of subobjects
	*
	* @var		integer	$draw_y
	* @access	private
	*/
	var $draw_y;

	/**
	* Alignment for the object
	*
	* @var		string	$alignment
	* @access	private
	*/
	var $alignment = "left";

	/**
	* Rotation for the object
	*
	* @var		float	$phi
	* @access	private
	*/
	var $phi = 0.0;

	/**
	* Text-Leading
	*
	* @var		integer	$leading
	* @access	private
	*/
	var $leading;

	/**
	* Linecolor
	*
	* @var		array	$linecolor
	* @access	private
	*/
	var $linecolor = array( 0, 0, 0 );

	/**
	* Fillcolor
	*
	* @var		string	$fillcolor
	* @access	private
	*/
	var $fillcolor = "";

	/**
	* pc4p_object constructor
	*
	* Checks, if we have a valid parent
	*
	* @param	object	pc4p_page	&$parent	Valid values are pc4p_page, pc4p_box, pc4p_table and pc4p_object
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_object( &$parent )
	{
		$accepted_classes = array( "pc4p_page", "pc4p_object", "pc4p_box" );

		// Checks, if the class has an allowed parent-Class
		if ( in_array( get_class( $parent ), $accepted_classes ) ) {
			// If yes, get PDF-Pointer
			$this->pdfp = &$parent->pdfp;
			$this->pdfobj = &$parent->pdfobj;
		}
		elseif ( get_class( $parent ) == "pc4p_table" && get_class( $this ) == "pc4p_object" ) {
			// Okay, we're called in a table as pc4p_object
			$this->pdfp = &$parent->pdfp;
			$this->pdfobj = &$parent->pdfobj;
		}
		elseif ( !is_object( $parent ) ) {
			die( "Error: ".get_class( $this )."(".__LINE__.") - parent is not an object." );
		}
		else {
			die( "Error: ".get_class( $this )."(".__LINE__.") - ".get_class( $parent )." not allowed as parent." );
		}
	}

	/**
	* Calculates the new coordinates after a rotation
	*
	* Simple rotation for a matrix. Can be found in any general book about linear algebra.
	*
	* @param	float	$x	X-coordinate
	* @param	float	$y	Y-coordinate
	* @param	float	$phi	Angle for the rotation
	* @return	array	$ret	Array containing the new [ "x" ] and [ "y" ]
	* @access	private
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_rot( $x, $y, $phi ) {
		$ret = array();
		$phi = deg2rad( $phi );
		$cosphi = cos( $phi );
		$sinphi = sin( $phi );
		$ret[ "x" ] = $x * $cosphi - $y * $sinphi;
		$ret[ "y" ] = $x * $sinphi + $y * $cosphi;
		return $ret;
	}

	/**
	* Calculates the new dimensions after a rotation
	*
	* Uses pc4p_calc_rot to determine the new coordinates for every corner of the object. Then it determines the new dimensions after a rotation.
	*
	* @param	float	$width	Width of the object
	* @param	float	$height	Height of the object
	* @param	float	$phi	Angle for the rotation
	* @return	array	$ret	Array containging the new [ "width" ] and [ "height" ]
	* @access	private
	* @see		pc4p_object::pc4p_calc_rot()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_rot_dim( $width, $height, $phi ) {
		$ret = array();
		$p2 = $this->pc4p_calc_rot( $width, 0, $phi );
		$p3 = $this->pc4p_calc_rot( 0, $height, $phi );
		$p4 = $this->pc4p_calc_rot( $width, $height, $phi );
		$ret[ "width" ]  = max( 0, $p2[ "x" ], $p3[ "x" ], $p4[ "x" ] ) - min( 0, $p2[ "x" ], $p3[ "x" ], $p4[ "x" ] );
		$ret[ "height" ] = max( 0, $p2[ "y" ], $p3[ "y" ], $p4[ "y" ] ) - min( 0, $p2[ "y" ], $p3[ "y" ], $p4[ "y" ] );
		return $ret;
	}

	/**
	* Sets the rotation for the object
	*
	* Accepts an angle, so that the object can be rotated
	*
	* @param	float	$phi	Angle for the rotation
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_rot( $phi ) {
		if ( is_numeric( $phi ) ) {
			$this->phi = $phi;
		}
	}

	/**
	* Sets the margins for the object
	*
	* As opposed to a page, here the margins are on the _inside_ the object. The array should look like array( "top" => top, "bottom" => bottom, "left" => left, "right" => right )
	*
	* @param	mixed	$margin	Array containing the margins for the object or an integer to set all margins
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_margin( $margin )
	{
		if ( is_array( $margin ) ) {
			foreach ( $margin as $mkey => $mvalue ) {
				if ( in_array( $mkey, array( "top", "bottom", "left", "right" ) ) ) {
					$this->margin[ $mkey ] = $mvalue;
				}
			}
		}
		elseif ( is_integer( $margin ) ) {
			foreach ( $this->margin as $mkey => $mvalue ) {
					$this->margin[ $mkey ] = $margin;
			}
		}
	}

	/**
	* Sets the width for the object
	*
	* You can set a default width for an object
	*
	* @param	mixed	$width	Width for the object
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_width( $width )
	{
		$this->width = $width;
	}

	/**
	* Sets the alignment for this object.
	*
	* The same functionality as in every other document-software
	*
	* @param	string	$alignment	Valid values are "left", "center" and "right"
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_alignment( $alignment = "left" )
	{
		$accepted_values = array( "left", "center", "right" );
		if ( in_array( strtolower( $alignment ), $accepted_values ) ) {
			$this->alignment = $alignment;
		}
		else {
			die( "Error: ".get_class( $this )."(".__LINE__.") - no such alignment: ".$alignment."." );
		}
	}

	/**
	* Sets the textleading for text-subobjects.
	*
	* You can define a spacing between each line of text
	*
	* @param	integer	$leading	Please use reasonable values, according to the fontsize
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_textleading( $leading )
	{
		if ( !empty( $leading ) ) {
			$this->leading = $leading;
		}
	}

	/**
	* Sets the color of the lines in this objects.
	*
	* You can define a linecolor here or reset the linecolor to default, if you don't give a parameter
	*
	* @param	string	$linecolor	You can use one of the 16 standardcolors from HTML or the hex-notation (#rrggbb)
	* @access	public
	* @see		pc4p_main::pc4p_check_color()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_linecolor( $linecolor )
	{
		$this->linecolor = pc4p_main::pc4p_check_color( $linecolor, "line" );
	}

	/**
	* Sets the color for a fill of this objects.
	*
	* You can define a fillcolor here or reset the fillcolor to default, if you don't give a parameter
	*
	* @param	string	$fillcolor	You can use one of the 16 standardcolors from HTML or the hex-notation (#rrggbb)
	* @access	public
	* @see		pc4p_main::pc4p_check_color()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_fillcolor( $fillcolor )
	{
		$this->fillcolor = pc4p_main::pc4p_check_color( $fillcolor, "fill" );
	}

	/**
	* Makes a real copy from an object and all subobjects
	*
	* This function is _ugly_ and I only wrote it to accomodate the perverse handling of references
	*
	* @param	object	pc4p_object	$object	The object you want to copy
	* @return	object	pc4p_object	$copy	The copy of the object
	* @access	private
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_make_objcopy( $object )
	{
		// As I said, this is ugly...nothing to see here, go on...
		$copy = $object;

		if( get_class( $copy ) == "pc4p_table" ) {
			unset( $copy->cell );   
			$copy->cell = array();  
			foreach( $object->cell as $ckey => $cvalue ) {
				$copy->cell[ $ckey ] = array();
				foreach( $cvalue as $rkey => $rvalue ) {
					$copy->cell[ $ckey ][ $rkey ] = pc4p_object::pc4p_make_objcopy( $rvalue );
				}
			}
		}
		else {
			unset( $copy->children );
			if( is_array( $object->children ) ) {
				$copy->children = array();
				foreach( $object->children as $child ) {
					$copy->children[] = pc4p_object::pc4p_make_objcopy( $child );
				}
			}
			if( isset( $object->borders ) && is_array( $object->borders ) ) {
				$copy->borders = array();
				foreach( $object->borders as $borderkey => $border ) {
					$copy->borders[ $borderkey ] = pc4p_object::pc4p_make_objcopy( $border );
				}
			}
		}

		return $copy;
	}

	/**
	* Draw-function for the object
	*
	* Calls the pc4p_draw_children
	*
	* @access	private
	* @see		pc4p_object::pc4p_draw_children()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw()
	{
		$this->pc4p_draw_children();

		unset( $this->pdfobj );
	}

	/**
	* Draw-children-function for the object
	*
	* Calls the draw function for each child in the children-array
	*
	* @access	private
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw_children()
	{
		for ( $i = 0; $i < sizeof( $this->children ); $i++) {
			$this->children[$i]->pc4p_draw();
		}
	}

	/**
	* Calls the calc_offset function in all children, sets its own offsets
	*
	* Checks for a already set width, watches for alignments
	*
	* @param	object	pc4p_page	&$parent	Parent of the object
	* @return	integer	$this->height	Calculated height of the object
	* @access	private
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_offset( &$parent )
	{
		if ( get_class( $parent ) == "pc4p_page" ) {
			$this->pagewidth = $parent->width;
			$this->pageheight = $parent->height;
		}
		else {
			$this->pagewidth = $parent->pagewidth;
			$this->pageheight = $parent->pageheight;
		}

		// Check, if someone set the width externally and as a relative value
		if ( isset( $this->width ) && preg_match( "/^(\d+)%$/", $this->width, $width ) ) {
			// Yes, someone did. So erase the % and calculate the width
			$this->width = ( $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] + $this->margin[ "left" ] + $this->margin[ "right" ] ) ) * $width[ 1 ] / 100.0;
		}

		// Check, if width is set, and if yes, if the width exceeds max width defined by parent
		if ( !isset( $this->width ) || $this->width <= 0 || $this->width > $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) ) {
			// If-Clause is true, so set width to max width
			$this->width = $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] );
		}

		$this->height = $this->margin[ "top" ];
		$this->act_x = $parent->draw_x;
		$this->act_y = $parent->draw_y;

		if ( $this->alignment == "center" ) {
			$this->draw_x = $this->act_x + $this->margin[ "left" ] + ( $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) - $this->width ) / 2;
		}
		elseif ( $this->alignment == "right" ) {
			$this->draw_x = $this->act_x + $this->margin[ "left" ] + $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) - $this->width;
		}
		else {
			$this->draw_x = $this->act_x + $this->margin[ "left" ];
		}

		$this->draw_y = $this->act_y + $this->margin[ "top" ];
		// Calculate the heights in the children and add them to our height
		for ( $i = 0; $i < sizeof( $this->children ); $i++) {
			$child_height = $this->children[$i]->pc4p_calc_offset( $this );
			$this->height += $child_height;
			$this->draw_y += $child_height;
		}
		$this->draw_y += $this->margin[ "bottom" ];
		$this->height += $this->margin[ "bottom" ];

		return $this->height;
	}
}
?>
