<?php
/**
* Supplies pagehandling ability
*
* In this object everything concerning the page is stored. Every page can have its own dimensions, background-images, etc. pp. Most important function in this object, is the ability to move objects from one page to the next, when the current page is full. This is called page-breaking.
*
* @author	Alexander Wirtz <alex@pc4p.net>
* @package	pc4p
* @version	$Id: pc4p_page.inc,v 1.60 2002/08/12 11:09:58 eru Exp $
*/
class pc4p_page
{
	/**
	* Array of all children of this object
	*
	* @var		array	$children
	* @access	private
	*/
	var $children = array();

	/**
	* PDF-Pointer used for all pdf_*
	*
	* @var		integer	$pdfp
	* @access	private
	*/
	var $pdfp;

	/**
	* The main PDF-Object
	*
	* @var		object pc4p_main	$pdfobj
	* @access	private
	*/
	var $pdfobj;

	/**
	* Should PC4P always create a new page on pagebreak?
	*
	* @var		integer	$newpage
	* @access private
	*/
	var $newpage = 0;

	/**
	* Width of the page
	*
	* @var		integer	$width
	* @access	private
	*/
	var $width;

	/**
	* Height of the page
	*
	* @var		integer	$height
	* @access	private
	*/
	var $height;

	/**
	* Pagenumber in the PDF
	*
	* @var		integer	$pagenum
	* @access	private
	*/
	var	$pagenum;

	/**
	* Margins for the page
	*
	* @var		array	$margin
	* @access	private
	*/
	var $margin = array( "top" => 5, "bottom" => 5, "left" => 5, "right" => 5 );

	/**
	* x-Pos where the page really starts placing objects
	*
	* @var		integer	$act_x
	* @access	private
	*/
	var $act_x = 0;

	/**
	* y-Pos where the page really starts placing objects
	*
	* @var		integer	$act_y
	* @access	private
	*/
	var $act_y = 0;

	/**
	* x-Pos used when calculating the positions for the objects
	*
	* @var		integer	$draw_x
	* @access	private
	*/
	var $draw_x;

	/**
	* y-Pos used when calculating the positions for the objects
	*
	* @var		integer	$draw_y
	* @access	private
	*/
	var $draw_y;

	/**
	* Bookmarks for this page
	*
	* @var		array	$bookmarks
	* @access	private
	*/
	var $bookmarks	= array();

	/**
	* Color of the background
	*
	* @var		string	$bg_color
	* @access	private
	*/
	var $bg_color = "";

	/**
	* Name of the background image
	*
	* @var		string	$bg_image
	* @access	private
	*/
	var $bg_image;

	/**
	* Type of the background image
	*
	* @var		string	$bgi_type
	* @access	private
	*/
	var $bgi_type;

	/**
	* x-Pos for the background image
	*
	* @var		float	$bgi_xoff
	* @access	private
	*/
	var $bgi_xoff;

	/**
	* y-Pos for the background image
	*
	* @var		float	$bgi_yoff
	* @access	private
	*/
	var $bgi_yoff;

	/**
	* Scale for the background image
	*
	* @var		float	$bgi_scale
	* @access	private
	*/
	var $bgi_scale;

	/**
	* Header of the page
	*
	* @var		object	pc4p_object	$header
	* @access	public
	*/
	var $header;

	/**
	* The height of the header, gets saved for following pages
	*
	* @var		float	$headerheight
	* @access	private
	*/
	var $headerheight = 0.0;

	/**
	* Footer of the page
	*
	* @var		object	pc4p_object	$footer
	* @access public
	*/
	var $footer;

	/**
	* The height of the footer, gets saved for following pages
	*
	* @var		float	$footerheight
	* @access	private
	*/
	var $footerheight = 0.0;

	/**
	* pc4p_page constructor
	*
	* Checks, if the dimensions for the page is acceptable and sets the PDF-pointer for the page.
	*
	* @param	object	pc4p_page	&$parent	MUST be a PDF-Object
	* @param	string	$format		Accepts a standard document-format or "user" for customized dimensions
	* @param	string	$orientation Accepts "portrait" ("p") or "landscape" ("l") as parameters, value is ignored, when customized dimensions are used
	* @param	string	$dimensions	If $format is "user", this parameter accepts a WIDTHxHEIGHT notation
	* @access	private
	* @see		pc4p_init::&pc4p_create_object(), pc4p_main::pc4p_open_pdf(), pc4p_object::pc4p_set_margin()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_page( &$parent, $format, $orientation = "p", $dimensions = "" )
	{
		$accepted_formats = array( "a0" => "2380x3368", "a1" => "1684x2380", "a2" => "1190x1684", "a3" => "842x1190", "a4" => "595x842", "a5" => "421x595", "a6" => "297x421", "b5" => "501x709", "letter" => "612x792", "legal" => "612x1008", "ledger" => "1224x792", "p11x17" => "792x1224");
		$accepted_classes = array( "pc4p_main" );

		// Checks, if the class has an allowed parent-Class
		if ( in_array( get_class( $parent ), $accepted_classes ) ) {
			// If yes, get PDF-Pointer
			$this->pdfp = &$parent->pdfp;
			$this->pdfobj = &$parent;
			$this->newpage = &$parent->newpage;
		}
		else {
			die( "Error: ".get_class( $this )."(".__LINE__.") - ".get_class( $parent )." not allowed as parent." );
		}

		// Set page format
		if ( in_array( strtolower( $format ), array_keys( $accepted_formats ) ) ) {
			// Ok, we have a standard format
			$xy = explode( "x", $accepted_formats[ $format ] );
			if ( $orientation == "l" || $orientation == "landscape" ) {
				$this->width = $xy[1];
				$this->height = $xy[0];
			}
			else {
				$this->width = $xy[0];
				$this->height = $xy[1];
			}
		}
		elseif ( $format == strtolower( "user" ) ) {
			// Homemade page. Set dimensions accordingly
			// Check on $orientation for BC-purposes only!!! Subject to removal!!!
			if ( preg_match( "/^(\d+)x(\d+)$/i", $dimensions, $xy ) || preg_match( "/^(\d+)x(\d+)$/i", $orientation, $xy ) ) {
				$this->width = $xy[1];
				$this->height = $xy[2];
			}
			else {
				die( "Error: ".get_class( $this )."(".__LINE__.") - dimensions not set." );
			}
		}
		else {
			die( "Error: ".get_class( $this )."(".__LINE__.") - unknown format: ".$format."." );
		}

		// I moved the call to pdf_open into this object, so that we can set parameters,
		// which belong to the object scope. I don't like this, but it won't work otherwise
		if ( !$parent->pdf_opened ) {
			$parent->pc4p_open_pdf();
		}

		// use this dummy as object for anchoring the header and the footer
		$dummy = new pc4p_object( $this );
		$this->header = &pc4p_create_object( $dummy );
		$this->footer = &pc4p_create_object( $dummy );
		$this->header->pc4p_set_margin( 5 );
		$this->footer->pc4p_set_margin( 5 );
	}

	/**
	* Changes the behaviour after a pagebreak
	*
	* Sets if a new page after a pagebreak shall be created, or if a already created page shall be used
	*
	* @param	mixed	$newpage
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_newpage( $newpage )
	{
		if( $newpage > 0 || $newpage == "yes" || $newpage == "y" ) {
			$this->newpage = 1;
		}
		else {
			$this->newpage = 0;
		}
	}

	/**
	* Sets the margins for the page
	*
	* A Page can have margins to space the objects from the, well, paper-border. The array should look like array( "top" => top, "bottom" => bottom, "left" => left, "right" => right ) or be an integer value
	*
	* @param	mixed	$margin	Array containing the margins for the page or an integer to set all margins
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_margin( $margin )
	{
		if ( is_array( $margin ) ) {
			foreach ( $margin as $mkey => $mvalue ) {
				if ( in_array( $mkey, array( "top", "bottom", "left", "right" ) ) ) {
					$this->margin[ $mkey ] = $mvalue;
				}
			}
		}
		elseif ( is_integer( $margin ) ) {
			foreach ( $this->margin as $mkey => $mvalue ) {
					$this->margin[ $mkey ] = $margin;
			}
		}
	}

	/**
	* Sets a bookmark for this page
	*
	* This is one of the hypertext-functions of PDF. You can link pages to create a content-overview. If no parent is set, a new root will be created.
	*
	* @param	string	$text		Description for the bookmark
	* @param	integer	$parent		Parent node, if $parent == 0 a new root will be created
	* @param	integer	$open		Triggers if the children get folded out
	* @return	integer	$identifier	New node-identifier
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_add_bookmark( $text, $parent = 0, $open = 1 )
	{
		if ( !is_numeric( $parent ) ) {
			$parent = 0;
		}
		if ( $open ) {
			$open = 1;
		}
		else {
			$open = 0;
		}

		$identifier = $this->pdfobj->bookmark_identifier++;
		$this->bookmarks[ $identifier ] = array( "text" => $text, "parent" => $parent, "open" => $open );
		$this->pdfobj->bookmarks[ $identifier ] = 0;
		return $identifier;
	}

	/**
	* Sets the background color for this page
	*
	* You can define a background color here or reset the color to default, if you don't give a parameter
	*
	* @param	string	$bgcolor	You can use one of the 16 standardcolors from HTML or the hex-notation (#rrggbb)
	* @access	public
	* @see		pc4p_main::pc4p_check_color()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_bgcolor( $bgcolor )
	{
		$this->bg_color = pc4p_main::pc4p_check_color( $bgcolor, "fill" );
	}

	/**
	* Sets a background image for this page
	*
	* You can define an image, which is used as a background-image for the page. Needed parameters are the image-name, the type of the image (gif, jpg, png, etc. ) and the exact location, where this image shall be placed. The fourth parameter is for scaling the image.
	*
	* @param	string	$imgfile	Name of the bg-image
	* @param	string	$imgtype	Type of the bg-image
	* @param	integer	$xoffset	X-offset of the bg-image ( 0 <= x <= page-width )
	* @param	integer $yoffset	Y-offset of the bg-image ( 0 <= y <= page-height )
	* @param	float	$scale		Scale of the bg-image
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_bgimage( $imgfile, $imgtype, $xoffset = 0, $yoffset = 0, $scale = 1.0 )
	{
		// Check for correct and supported image-type
		switch ( strtolower( $imgtype ) ) {
			case "png" :	$this->bgi_type = "png";
							break;
			case "gif" :	$this->bgi_type = "gif";
							break;
			case "tiff" :
			case "tif" :	$this->bgi_type = "tiff";
							break;
			case "jpeg" :
			case "jpg" :	$this->bgi_type = "jpeg";
							break;
			default:		die( "Error: ".get_class( $this )."(".__LINE__.") - this imagefiletype is not supported: ".$imgtype."." );
		}
		if ( file_exists( $imgfile ) ) {
			// Everything's cool
			$this->bg_image = $imgfile;
		}
		else {
			die( "Error: ".get_class( $this )."(".__LINE__.") - image not found: ".$imgfile."." );
		}
		$this->bgi_xoff = $xoffset;
		$this->bgi_yoff = $yoffset;
		$this->bgi_scale = $scale;
	}

	/**
	* Draw-function for the page
	*
	* Begins the page with the dimensions, calls the draw for each child and closes the page. Called as last function in the class.
	*
	* @param	object	pc4p_main	&$parent
	* @access	public
	* @see		pc4p_object::pc4p_draw(), pc4p_page::pc4p_draw_children()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw( &$parent )
	{
		if ( is_array( $this->bg_color ) ) {
			pdf_save( $this->pdfp );
			pdf_setcolor( $this->pdfp, "both", "rgb", $this->bg_color[ 0 ], $this->bg_color[ 1 ], $this->bg_color[ 2 ], 0 );
			pdf_rect( $this->pdfp, 0, 0, $this->width, -$this->height );
			pdf_closepath_fill_stroke( $this->pdfp );
			pdf_restore( $this->pdfp );
		}
		if ( !empty( $this->bg_image ) ) {
			$pim = pdf_open_image_file( $this->pdfp, $this->bgi_type, $this->bg_image );
			$this->bgi_yoff += pdf_get_value( $this->pdfp, "imageheight", $pim ) * $this->bgi_scale;
			pdf_place_image( $this->pdfp, $pim, $this->bgi_xoff, -$this->bgi_yoff, $this->bgi_scale );
			pdf_close_image( $this->pdfp, $pim );
		}

		// Draw the header, children and footer
		$this->header->pc4p_draw();
		$this->pc4p_draw_children();
		$this->footer->pc4p_draw();
		// ...and close page
		pdf_end_page( $this->pdfp );

		unset( $this->pdfobj );
	}

	/**
	* Draw-children-function for the page
	*
	* Calls the draw function for each child in the children-array
	*
	* @access	private
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw_children()
	{
		for ( $i = 0; $i < sizeof( $this->children ); $i++) {
			$this->children[$i]->pc4p_draw();
		}
	}

	/**
	* Moves the children from a given position onto the next page
	*
	* Checks if a next page is available and if a move onto this page is allowed. If not, creates a new one. Moves remaining children onto the next page.
	*
	* @param	object	pc4p_main	&$parent	Parent of this page, must be a PDF-Object
	* @param	integer	$position	Position in the children-array, from which on the children get moved onto the next page
	* @param	float	$child_height	Height of the child at $position
	* @access	private
	* @see		pc4p_page::pc4p_set_margin(), pc4p_page::pc4p_set_bgimage(), pc4p_object::pc4p_make_objcopy()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_make_pagebreak( &$parent, $position, $child_height ) {
		// check if there's a next page
		if ( $this->newpage || !is_object( $parent->next_page ) ) {
			// No, there isn't. Create a new one!
			// Check if object fits into page, if it's a table or text, don't check, just move it
			if ( $child_height > ( $this->height - ( $this->headerheight + $this->margin[ "top" ] + $this->margin[ "bottom" ] + $this->footerheight ) ) && get_class( $this->children[ $position ] ) != "pc4p_table"  &&  get_class( $this->children[ $position ] ) != "pc4p_text" ) {
				die( "Error: ".get_class( $this->children[ $position ] )."(".__LINE__.") - Object is too large for current margins/dimensions." );
			}
			$next_page = &new pc4p_page( $parent, "user", $this->width."x".$this->height );
			if( is_object( $parent->next_page ) ) {
				for( $c = sizeof( $parent->children ); $c > $this->pagenum; $c-- ) {
					$parent->children[ $c ] = $parent->children[ $c - 1 ];
				}
				$parent->children[ $this->pagenum ] = &$next_page;
			}
			else {
				$parent->children[] = &$next_page;
			}
			$next_page->pc4p_set_margin( $this->margin );
			$next_page->bg_color = $this->bg_color;
			if ( isset( $this->bg_image ) ) {
				$next_page->pc4p_set_bgimage( $this->bg_image, $this->bgi_type, $this->bgi_xoff, $this->bgi_yoff, $this->bgi_scale );
			}

			// Copy header and footer to the next page
			$next_page->header = pc4p_object::pc4p_make_objcopy( $this->header );
			$next_page->headerheight = $this->headerheight;
			$next_page->footer = pc4p_object::pc4p_make_objcopy( $this->footer );
			$next_page->footerheight = $this->footerheight;

			// Move the remaining objects to the next page.
			$next_page->children = array_merge( array_splice( $this->children, $position ), $next_page->children );
		}
		else {
			// Yes, there is.
			// Check if object fits into page, if it's a table, don't check, just move it
			if ( $child_height > ( $parent->next_page->height - ( $parent->next_page->headerheight + $parent->next_page->margin[ "top" ] + $parent->next_page->margin[ "bottom" ] + $parent->next_page->footerheight ) ) && get_class( $this->children[ $position ] ) != "pc4p_table" && get_class( $this->children[ $position ] ) != "pc4p_text" ) {
				die( "Error: ".get_class( $this->children[ $i ] )."(".__LINE__.") - Object is too large for margins/dimensions of the next page." );
			}
			// Move the remaining objects in front of the objects on the next page.
			$parent->next_page->children = array_merge( array_splice( $this->children, $position ), $parent->next_page->children );
		}
	}
	
	/**
	* Calculates the offsets in this page for every object contained in it
	*
	* Calls the calc_offset function in all children. Checks, if the current object still fits onto the page. If not, it checks if there's a following page; if not, it creates one, if yes it uses the existing. Moves the overlapping objects to the new page. In one word: Here's the pagebreak function!!!
	*
	* @param	object	pc4p_main	&$parent	Parent of this page, must be a PDF-Object
	* @access	private
	* @see		pc4p_object::pc4p_set_margin(), pc4p_object::pc4p_calc_offset(), pc4p_page::pc4p_make_pagebreak(), pc4p_table::pc4p_calc_row_offset(), pc4p_table::pc4p_split_table(), pc4p_text::pc4p_calc_offset(), pc4p_text::pc4p_split_text()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_offset( &$parent )
	{
		// Initialize page and set (0,0) to the upper left corner
		pdf_begin_page( $this->pdfp, $this->width, $this->height );
		pdf_translate( $this->pdfp, 0, $this->height );

		// Add bookmarks for this page
		foreach ( $this->bookmarks as $identifier => $values ) {
			$this->pdfobj->bookmarks[ $identifier ] = pdf_add_bookmark( $this->pdfp, $values[ "text" ], $this->pdfobj->bookmarks[ $values[ "parent" ] ], $values[ "open" ] );
		}

		if ( defined( "PC4PGRID" ) ) {
			pdf_save( $this->pdfp );
			pdf_setcolor( $this->pdfp, "stroke", "rgb", 0.85882352941, 0.97647058824, 0.97647058824, 0 );
			pdf_setlinewidth( $this->pdfp, 0.1 );
			for ( $x = PC4PGRID; $x < $this->width; $x += PC4PGRID ) {
				pdf_moveto( $this->pdfp, $x, 0 );
				pdf_lineto( $this->pdfp, $x, -$this->height );
			}
			for ( $y = PC4PGRID; $y < $this->height; $y += PC4PGRID ) {
				pdf_moveto( $this->pdfp, 0, -$y );
				pdf_lineto( $this->pdfp, $this->width, -$y );
			}
			pdf_closepath_stroke( $this->pdfp );
			pdf_restore( $this->pdfp );
		}

		// Save this pages margin, so that we can restore it later, then set it to zero for proper
		// width calculation for header and footer
		$margin_left = $this->margin[ "left" ];
		$margin_right = $this->margin[ "right" ];
		$this->margin[ "left" ] = 0;
		$this->margin[ "right" ] = 0;

		// First determine the height of the header
		if ( $this->headerheight <= 0 ) {
			if ( sizeof( $this->header->children ) == 0 ) {
				$this->header->pc4p_set_margin( 0 );
			}
			$this->draw_x = 0;
			$this->draw_y = 0;
			$this->headerheight = $this->header->pc4p_calc_offset( $this );
		}
		// Now we have to determine the height of the footer. As we don't know its height yet,
		// we have to create a dummy object and use it as parent for the footer
		if ( $this->footerheight <= 0 ) {
			$dummy = new pc4p_object( $this );
			$dummy->draw_x = 0;
			$dummy->draw_y = 0;
			$dummy->pagewidth = $this->width;
			$dummy->pageheight = $this->height;
			$dummy->width = $this->width;
			$this->footerheight = $this->footer->pc4p_calc_offset( $dummy );
			// Now we know the height of the footer and calculate its coordinates for the page
			if ( sizeof( $this->footer->children ) == 0 ) {
				$this->footer->pc4p_set_margin( 0 );
			}
			$this->draw_x = 0;
			$this->draw_y = $this->height - $this->footerheight;
			$this->footerheight = $this->footer->pc4p_calc_offset( $this );
		}
		// Restore the margins
		$this->margin[ "left" ] = $margin_left;
		$this->margin[ "right" ] = $margin_right;

		// We continue below the header
		$this->draw_x = $this->margin[ "left" ];
		$this->draw_y = $this->margin[ "top" ] + $this->headerheight;

		for ( $i = 0; $i < sizeof( $this->children); $i++ ) {
			$child_height = 0;
			// Check if the object is a table
			if ( get_class( $this->children[ $i ] ) == "pc4p_table" ) {
				// Yes! So calculate, how many rows fit onto the page
				$tmp_size = $this->draw_y + $this->children[ $i ]->margin[ "top" ];
				for ( $row = 0; $row < sizeof( $this->children[ $i ]->rownames ); $row++ ) {
					$row_size = $this->children[ $i ]->pc4p_calc_row_offset( $this, $row );
					// Check, if the row fits on a single page
					if ( $row_size > $this->height - ( $this->headerheight + $this->margin[ "top" ] + $this->children[ $i ]->margin[ "top" ] + $this->children[ $i ]->margin[ "bottom" ] + $this->margin[ "bottom" ] + $this->footerheight ) ) {
						die( "Error: ".get_class( $this->children[ $i ] )."(".__LINE__.") - Row ".$row." is too large for current margins/dimensions." );
					}
					// Check, if the row still fits onto the current page
					if ( $tmp_size + $row_size + $this->children[ $i ]->margin[ "bottom" ] + $this->margin[ "bottom" ] + $this->footerheight > $this->height ) {
						// It doesn't, so $row is the row, where the table is cut off
						// As the height was already increased internally, subtract the last row
						$this->children[ $i ]->height -= $row_size;
						$this->children = array_merge( array_slice( $this->children, 0, $i + 1 ), array( $this->children[ $i ]->pc4p_split_table( $row ) ), array_slice( $this->children, $i + 1 ) );
						$this->children[ $i ]->height += $this->children[ $i ]->margin[ "bottom" ];
						$child_height = $this->children[ $i ]->height;
						$i++;
						$this->pc4p_make_pagebreak( $parent, $i, $child_height );
						break;
					}
					else {
						// Ok, still enough space, add row and continue
						$tmp_size += $row_size;
					}
				}
				if ( $child_height == 0 ) {
					// Calculate the proper heights for the child
					$this->children[ $i ]->height += $this->children[ $i ]->margin[ "bottom" ];
					$child_height = $this->children[ $i ]->height;
				}
			}
			elseif ( get_class( $this->children[ $i ] ) == "pc4p_text" ) {
				// First calculate expected height of the text
				$child_height = $this->children[ $i ]->pc4p_calc_offset( $this );
				if ( $this->draw_y + $child_height + $this->margin[ "bottom" ] + $this->footerheight > $this->height ) {
					// Doesn't fit on the page...pagebreak
					// Calculate space left on the page and how many rows will fit into that space
					$space_left = $this->height - ( $this->draw_y + $this->children[ $i ]->margin[ "top" ] + $this->children[ $i ]->margin[ "bottom" ] + $this->margin[ "bottom" ] + $this->footerheight );
					$row_size = $this->children[ $i ]->fontsize + ( $this->children[ $i ]->leading - $this->children[ $i ]->fontsize );
					$row = (int) floor( $space_left / $row_size );
					// Split text at that row
					$this->children = array_merge( array_slice( $this->children, 0, $i + 1 ), array( $this->children[ $i ]->pc4p_split_text( $row ) ), array_slice( $this->children, $i + 1 ) );
					// Calculate new child height and that of the split-off child
					$child_height = $this->children[ $i ]->pc4p_calc_offset( $this );
					$i++;
					$this->pc4p_make_pagebreak( $parent, $i, $child_height );
				}
			}
			else {
				// It isn't...get the height for the current child and...
				$child_height = $this->children[ $i ]->pc4p_calc_offset( $this );
			}

			// ...check if it still fits onto the current page
			if ( $this->draw_y + $child_height + $this->margin[ "bottom" ] + $this->footerheight > $this->height ) {
				// No, it doesn't, so make a pagebreak
				$this->pc4p_make_pagebreak( $parent, $i, $child_height );
			}
			else  {
				// It fits! So increase the used height by the height of the object.
				$this->draw_y += $child_height;
			}
		}
		$this->draw_y += $this->margin[ "bottom" ];
	}
}
?>
